﻿/*!***********************************
插件名称：RCommon
当前版本：1.5.3
更新日期：2022.8.23
插件官网：http://www.rplugin.com
Copyright (c) 2022 RUI, Robert Wu
Released under the MIT license
*************************************/

// #region FunctionExtensions

interface Function {
    /**
     * 继承
     * @param {Function} superClass 父类
     */
    extend(superClass: Function): Function;
}

// #endregion

// #region StringExtensions

interface String {
    /**
     * 尝试将字符串转换成整数并返回，当转换失败时返回指定默认值
     * @param {number} defaultValue 指定默认值
     */
    parseInt(defaultValue: number): number;

    /**
     * 尝试将字符串转换成浮点数并返回，当转换失败时返回指定默认值
     * @param {number} defaultValue 指定默认值
     */
    parseFloat(defaultValue: number): number;

    /** 清除前置和后置的不可见字符 */
    trim(): string;
    /**
     * 清除前置和后置的指定字符
     * @param {string} target 指定字符
     */
    trim(target: string): string;

    /** 清除前置的不可见字符 */
    trimStart(): string;
    /**
     * 清除前置的指定字符
     * @param {string} target 指定字符
     */
    trimStart(target: string): string;

    /** 清除后置的不可见字符 */
    trimEnd(): string;
    /**
     * 清除后置的指定字符
     * @param {string} target 指定字符
     */
    trimEnd(target: string): string;

    /**
     * 使用指定正则替换所有匹配
     * @param {RegExp} reallyDo 指定正则对象
     * @param {string} replaceWith 要替换成的字符串
     */
    replaceAll(reallyDo: RegExp, replaceWith: string): string;
    /**
     * 使用指定正则表达式替换所有匹配（区分大小写）
     * @param {string} reallyDo 指定正则表达式
     * @param {string} replaceWith 要替换成的字符串
     */
    replaceAll(reallyDo: string, replaceWith: string): string;
    /**
     * 使用指定正则表达式替换所有匹配，并指定是否区分大小写
     * @param {string} reallyDo 指定正则表达式
     * @param {string} replaceWith 要替换成的字符串
     * @param {boolean} ignoreCase 是否忽略大小写
     */
    replaceAll(reallyDo: string, replaceWith: string, ignoreCase: boolean): string;

    /**
     * 将字符串转换成驼峰写法（去掉'-'，并把'-'后面的第一个字母改成大写）
     * 例：输入"background-image"，返回"backgroundImage"
     */
    camelize(): string;

    /** 将字符串中的首个字符（如果是英文字符的话）转换为大写 */
    capFirst(): string;

    /**
     * 进行HTML编码
     * 例：传入"<test>\n"，返回"&lt;test&gt;<br/>"
     */
    encodeHtml(): string;

    /**
     * 进行HTML解码
     * 例：传入"&lt;test&gt;<br/>"，返回"<test>\n"
     */
    decodeHtml(): string;

    /** 将字符串中的双引号编码成字符实体"&amp;quot;" */
    encodeQuot(): string;

    /** 进行escape编码 */
    escape(): string;

    /** 进行unescape解码 */
    unescape(): string;

    /**
     * 判断当前字符串是否以指定的字符串为开头
     * @param {string} target 指定字符串
     */
    startsWith(target: string): boolean;

    /**
     * 判断当前字符串是否以指定的字符串为结尾
     * @param {string} target 指定字符串
     */
    endsWith(target: string): boolean;

    /**
     * 当字符串长度超过指定最大长度时，截断溢出的部分，并添在末尾添加"..."
     * @param {number} maxlength 指定最大长度
     */
    ellipsis(maxlength: number): string;
    /**
     * 当字符串长度超过指定最大长度时，截断溢出的部分，并添在末尾添加ellipsisString
     * @param {number} maxlength 指定最大长度
     * @param {string} ellipsisString 当截断时要在末尾添加的字符串
     */
    ellipsis(maxlength: number, ellipsisString: string): string;
}

// #endregion

// #region NumberExtensions

interface Number {
    /**
     * 保留到小数点后2位（四舍五入）
     */
    round(): number;
    /**
     * 保留到小数点后指定位（四舍五入）
     * @param {number} roundDigitc 保留到小数点后多少位，默认两位
     */
    round(roundDigit: number): number;
    /**
     * 保留到小数点后指定位，并指定是否进行四舍五入
     * @param {number} roundDigitc 保留到小数点后多少位，默认两位
     * @param {boolean} fivein 是否进行四舍五入，默认进行
     */
    round(roundDigit: number, fivein: boolean): number;

    /** 判断该数字是否存在小数 */
    isDecimal(): boolean;
}

// #endregion

// #region ArrayExtensions

interface Array<T> {
    /**
     * 查找指定对象在数组中第一次出现的下标，使用强等于(===)判断
     * @param {T} obj 指定对象
     */
    indexOf(obj: T): number;

    /**
     * 查找符合指定过滤条件的第一个元素的下标
     * @param {(item: T) => boolean} fn 指定过滤条件
     */
    indexOfFn(fn: (item: T) => boolean): number;

    /**
     * 获取指定下标的元素，若不存在则返回指定默认值
     * @param {number} index 指定下标
     * @param {any} defaultValue 指定默认值
     */
    get(index: number, defaultValue: any): T;

    /**
     * 遍历数组对每一个元素执行指定函数，若函数返回false则停止遍历
     * @param {(this: T, index: number, item: T) => boolean | void} fn 指定函数
     */
    each(fn: (this: T, index: number, item: T) => boolean | void): this;

    /**
     * 无重复地添加一个元素到末尾
     * 若元素已存在则不进行操作
     * @param {T} obj 指定要添加的元素
     */
    add(obj: T): this;

    /**
     * 删除指定元素，使用强等于(===)判断
     * 若同时存在多个相同元素则会全部删除；若元素不存在则不进行操作，不会报错
     * @param {T} obj 指定要删除的元素
     */
    remove(obj: T): this;

    /**
     * 删除指定下标的元素
     * @param {number} index 指定下标
     */
    removeAt(index: number): this;

    /**
     * 获取第一个元素
     * @param {any} defaultValue 指定默认值
     */
    first(defaultValue: any): T;

    /**
     * 获取最后一个元素
     * @param {any} defaultValue 指定默认值
     */
    last(defaultValue: any): T;

    /**
     * 根据指定过滤条件筛选出符合条件的元素的数组
     * @param {(item: T) => boolean} fn 指定过滤条件
     */
    where(fn: (item: T) => boolean): Array<T>;

    /**
     * 根据指定过滤条件筛选出符合条件的元素的数组
     * @param {(item: T) => boolean} fn 指定过滤条件
     */
    select(fn: (item: T) => boolean): Array<T>;

    /**
     * 根据指定过滤条件筛选出符合条件的第一个元素
     * @param {(item: T) => boolean} fn 指定过滤条件
     * @param {any} defaultValue 当未找到任何元素时返回的默认值
     */
    selectFirst(fn: (item: T) => boolean, defaultValue: any): T;

    /**
     * 在指定下标处插入一个元素
     * @param {T} item 要插入的元素
     * @param {number} index 指定下标
     */
    insert(item: T, index: number): this;

    /**
     * 将当前数组中的每个元素处理成新的元素并返回新元素的数组
     * @param {(this: T, item: T, index: number, arr: this) => any} fn 将现有元素处理成新元素的处理函数，返回值视为新元素
     */
    map(fn: (this: T, item: T, index: number, arr: this) => any): this;

    /**
     * 将当前数组中的每个元素处理成新的元素并返回新元素的数组
     * @param {(this: T, item: T, index: number, arr: this) => any} fn 将现有元素处理成新元素的处理函数，返回值视为新元素
     */
    remap(fn: (this: T, item: T, index: number, arr: this) => any): this;

    /**
     * 指定一个判断条件，将符合该条件的所有元素移除
     * @param {(item: T) => boolean} fn 指定判断条件
     */
    removeBy(fn: (item: T) => boolean): this;
}

// #endregion

// #region jQueryExtensions

interface JQueryStatic {
    /**
     * 创建一个元素
     * @param {string} tagName 标签名称
     */
    create(tagName: string): JQuery;
    /**
     * 创建一个元素
     * @param {string} tagName 标签名称
     * @param {string} className 类名
     */
    create(tagName: string, className: string): JQuery;
    /**
     * 创建一个元素
     * @param {string} tagName 标签名称
     * @param {string} className 类名
     * @param {any} html HTML内容
     */
    create(tagName: string, className: string, html: any): JQuery;
}

interface JQuery {
    /**
     * 获取指定名称的属性的值，若不存在则返回空字符串
     * @param {string} name 指定名称
     */
    attrstr(name: string): string;

    /**
     * 根据指定值显示/隐藏元素
     * @param {any} value 需要显示(true)还是隐藏(false)
     */
    showhide(value: any): this;

    /**
     * 根据指定值添加/移除类名
     * @param {string} className 指定类名
     * @param {any} value 需要添加类名(true)还是移除类名(false)
     */
    addRemoveClass(className: string, value: any): this;

    /**
     * 判断指定的坐标是否在当前元素内
     * @param {number} x X坐标
     * @param {number} y Y坐标
     */
    isInside(x: number, y: number): boolean;
}

// #endregion

// #region RCommon

interface RCommonClass {
    /**
     * 在顶层window对象中查找指定名称的window对象，若找不到则返回null，若指定名称为空则直接返回顶层window
     * @param {string} name 指定名称
     */
    getWindowByName(name: string): Window;
    /**
     * 在指定window对象中查找指定名称的window对象，若找不到则返回null，若指定名称为空则直接返回顶层window
     * @param {Window} iwindow 指定window对象
     * @param {string} name 指定名称
     */
    getWindowByName(iwindow: Window, name: string): Window;

    /**
     * 遍历当前顶层页面及其内含的所有iframe页面执行操作
     * @param {(w: Window) => void | boolean} fn 要执行的动作
     */
    eachWindow(fn: (w: Window) => void | boolean): this;
    /**
     * 遍历指定页面及其内含的所有iframe页面执行操作
     * @param {Window} window 指定页面
     * @param {(w: Window) => void | boolean} fn 要执行的动作
     */
    eachWindow(window: Window, fn: (w: Window) => void | boolean): this;

    /** 生成4个16位（0~f）随机码 */
    random16_4(): string;

    /** 生成一个随机的GUID */
    guid(): string;

    /**
     * 获取指定元素的指定样式名称的最顶层值
     * 样式名称请使用标准css写法
     * 例：请使用"background-color"，不要使用"backgroundColor"
     * @param {Element} dom 指定元素的原生dom对象
     * @param {string} styleName 指定样式名称
     */
    style(dom: Element, styleName: string): string;

    /**
     * 获取分页数（当共有recordCount条记录，每页显示pageSize条记录时，计算一共有几页）
     * @param {number} recordCount 共有多少条记录
     * @param {number} pageSize 每页显示几条记录
     */
    getPage(recordCount: number, pageSize: number): number;

    /**
     * 判断一个输入值的数据类型
     * @param {any} input 输入值
     */
    getType(input: any): "undefinedOrNull" | "jquery" | "dom" | "array" | "simpleData" | "function" | "object";

    /**
     * 克隆一个数据
     * @param {any} input 要克隆的数据
     */
    clone(input: any): any;
    /**
     * 克隆一个数据，并指定复制设置
     * @param {any} input 要克隆的数据
     * @param {RCommonCloneSettings} settings 复制设置
     */
    clone(input: any, settings: RCommonCloneSettings): any;

    /**
     * 判断输入的参数是否为一个纯粹的对象
     * @param {any} obj 要判断的参数
     */
    isPlainObject(obj: any): boolean;
}

/** 提供一些常用辅助方法 */
declare const RCommon: RCommonClass;

/**
 * 遍历数组执行动作
 * @param {ArrayLike<T>} arr
 * @param {(item: T, index) => void | false} fn
 */
declare function foreach<T>(arr: ArrayLike<T>, fn: (item: T, index) => void | false): void;

interface RCommonCloneSettings {
    /** 是否需要自动为对象添加唯一标识key */
    addKeyToObject: boolean;
    /** 是否需要复制原生dom元素和jQuery对象 */
    cloneDom: boolean;
    /** 需要跳过处理的属性名 */
    skipKeys: Array<string>;
}

// #endregion

// #region RUrl

interface RUrlClass {
    /**
     * 从当前页面的URL地址中获取指定名称的URL参数的值
     * @param {string} name 指定名称
     */
    queryString(name: string): string;
    /**
     * 从指定URL地址中获取指定名称的URL参数的值
     * @param {string} url 指定URL地址
     * @param {string} name 指定名称
     */
    queryString(url: string, name: string): string;

    /**
     * 获取指定URL地址中的所有URL参数
     * @param {string} url 指定URL地址
     */
    getAllParams(url: string): object;

    /**
     * 使用指定对象的属性作为URL参数修改指定URL地址
     * @param {string} url 指定URL地址
     * @param {object} map 指定对象（若对象某属性值为null或undefined则视为移除该参数）
     */
    changeParam(url: string, map: object): string;
    /**
     * 修改指定URL地址中指定名称的URL参数的值为指定值，若不存在该参数则将自动添加
     * @param {string} url 指定URL地址
     * @param {string} name 指定名称
     * @param {string} value 指定值（若值为null或undefined则视为移除该参数）
     */
    changeParam(url: string, name: string, value: string): string;

    /**
     * 获取当前全部的Hash参数
     */
    hash(): object;
    /**
     * 获取指定名称的Hash参数的值
     * @param {string} name 指定名称
     */
    hash(name: string): string;
    /**
     * 设置指定名称的Hash参数为指定值
     * @param {string} name 指定名称
     * @param {string} value 指定值
     */
    hash(name: string, value: string): this;
    /**
     * 批量设置多个Hash参数的值
     * @param {object} data 一个包含键值对的对象
     */
    hash(data: object): this;
}

/** 提供一些用于URL操作的方法 */
declare const RUrl: RUrlClass;

// #endregion

// #region RCookie

interface RCookieClass {
    /** 是否要对来自localhost的请求进行端口区分的特殊处理（默认是） */
    distinguishLocalhostPort: boolean;

    /**
     * 获取根据localhost进行端口区分后的新的键名
     * @param {string} name 指定在区分端口前的键名
     */
    getDistinguishedName(name: string): string;

    /**
     * 添加指定名称的cookie值，对value进行escape编码，有效时间为session时间，添加到根目录
     * @param {string} name 指定名称
     * @param {string} value 指定值
     */
    add(name: string, value: string): this;
    /**
     * 添加指定名称的cookie值，并自定义额外设置选项
     * @param {string} name 指定名称
     * @param {string} value 指定值
     * @param {AddingCookieOptions} options 额外设置选项
     */
    add(name: string, value: string, options: AddingCookieOptions): this;

    /**
     * 获取指定名称的cookie的值，获取不到时返回空字符串
     * @param {string} name 指定名称
     */
    get(name: string): string;

    /**
     * 删除指定名称的cookie
     * @param {string} name 指定名称
     */
    remove(name: string): this;

    /** 获取当前域所有的cookie的键值对的数组 */
    getall(): Array<KeyValuePair<string, string>>;

    /** 删除当前域所有的cookie */
    clear(): this;
}

interface KeyValuePair<TKey, TValue> {
    name: TKey,
    value: TValue
}

/** 提供一些用于Cookie操作的方法 */
declare const RCookie: RCookieClass;

interface AddingCookieOptions {
    /** 有效时长（分钟数）（当未设置时有效时长为session时长） */
    minutes: number;
    /** 是否需要对值进行escape操作（默认是） */
    doEscape: boolean;
    /** 指定目录（默认为"/"根目录） */
    path: string;
    /** 指定域 */
    domain: string;
}

// #endregion

// #region RLocalStorage

interface RLocalStorageClass {
    /** 是否要对来自localhost的请求进行端口区分的特殊处理（默认是） */
    distinguishLocalhostPort: boolean;

    /**
     * 获取根据localhost进行端口区分后的新的键名
     * @param {string} name 指定在区分端口前的键名
     */
    getDistinguishedName(name: string): string;

    /**
     * 添加指定名称的localStorage数据
     * @param {string} name 指定名称
     * @param {string} value 数据
     */
    add(name: string, value: string): this;

    /**
     * 删除指定名称的localStorage数据
     * @param {string} name 指定名称
     */
    remove(name: string): this;

    /**
     * 获取指定名称的localStorage数据，获取不到时返回空字符串
     * @param {string} name 指定名称
     */
    get(name: string): string;

    /** 获取当前域所有的localStorage的键值对的数组 */
    getall(): Array<KeyValuePair<string, string>>;

    /** 删除当前域所有的localStorage数据 */
    clear(): this;
}

/** 提供一些用于LocalStorage操作的方法 */
declare const RLocalStorage: RLocalStorageClass;

// #endregion

// #region RDebug

interface RDebugClass {
    /** 上上次记录的时间 */
    _t1: number;

    /** 上次记录的时间 */
    _t2: number;

    /** 记录时间 */
    lt(): this;

    /** 获取耗时（最近两次记录的时间的时间差） */
    t(): number;

    /** 获取耗时的提示文字 */
    ts(): string;

    /** 弹框提示耗时 */
    tsa(): this;

    /**
     * 快速输出一段console日志
     * @param {string} text 日志内容
     */
    log(text: string): this;
    /**
     * 快速输出一段console日志
     * @param {string} text 日志内容
     * @param {string} color 指定颜色
     */
    log(text: string, color: string): this;
}

/** 提供一些用于调试的方法 */
declare const RDebug: RDebugClass;

// #endregion

// #region RDomMaker

interface RDomMakerMapItem {
    /** 标签名称【默认为"div"】 */
    tag: string;
    /** 加入到指定名称的元素的最后（输入"jdom"将视为主元素） */
    appendTo: string;
    /** 加入到指定名称的元素的最前（输入"jdom"将视为主元素） */
    prependTo: string;
    /** 插入到指定名称的元素的后面（输入"jdom"将视为主元素） */
    insertBefore: string;
    /** 插入到指定名称的元素的前面（输入"jdom"将视为主元素） */
    insertAfter: string;
    /** 在创建元素时验证元素是否已存在，若函数返回一个jQuery对象则视为已存在，此时不会执行元素创建动作，而是直接将返回的对象作为创建结果 */
    checkexist: (name: string, data: RDomMakerMapItem, doms: object, settings: RDomMakerMakingSettings) => any;
    /** 当生成设置中设置了统一类名前缀时，是否要为其添加前缀【默认是】 */
    prefix: boolean;
    /** 当生成设置中设置了统一类名后缀时，是否要为其添加后缀【默认是】 */
    postfix: boolean;
    /** 是否要在类名的前面添加RDomMaker的name【默认是】 */
    addNameToClass: boolean;
    /** 为元素添加额外的类名 */
    extraclass: string;
    /** 为元素添加初始html内容 */
    html: any;
    /** 是否初始隐藏该元素【默认否】 */
    hidden: boolean;
    /** 为元素添加初始属性 */
    attrs: object;
    /** 是否需要在initDoms时自动创建该元素【默认是】 */
    initcreate: boolean;
}

interface RDomMakerMakingSettings {
    /** 主元素 */
    root: JQuery;
    /** 统一类名前缀 */
    prefixname: string;
    /** 统一类名后缀 */
    postfixname: string;
    /** 每当元素生成完成时要执行的回调函数 */
    makingDom: (jdom: JQuery, name: string, data: RDomMakerMapItem) => void;
}

declare class RDomMaker {
    /** 名称 */
    name: string;
    /** 图纸内容 */
    map: { [key: string]: RDomMakerMapItem; };

    /**
     * 元素生成器
     * @param {string} name 名称
     * @param {{ [key: string]: RDomMakerMapItem; }} dommap 图纸内容
     */
    constructor(name: string, dommap: { [key: string]: RDomMakerMapItem; });

    /**
     * 遍历map对每一项执行指定的函数
     * @param {(name: string, data: object) => void} fn 要执行的函数
     */
    eachDoms(fn: (name: string, data: RDomMakerMapItem) => void): this;

    /**
     * 初始化map中的所有项到容器
     * @param {object} doms 用于存放元素的容器（一个对象）
     * @param {RDomMakerMakingSettings} settings 生成设置
     */
    initDoms(doms: object, settings: RDomMakerMakingSettings): this;

    /**
     * 创建指定名称的元素到容器
     * @param {string} name 指定名称
     * @param {object} doms 用于存放元素的容器（一个对象）
     * @param {RDomMakerMakingSettings} settings 生成设置
     */
    createDom(name: string, doms: object, settings: RDomMakerMakingSettings): JQuery;

    /**
     * 创建指定名称数组的元素到容器
     * @param {Array<string>} names 指定名称数组
     * @param {object} doms 用于存放元素的容器（一个对象）
     * @param {RDomMakerMakingSettings} settings 生成设置
     */
    createDoms(names: Array<string>, doms: object, settings: RDomMakerMakingSettings): this;

    /**
     * 创建指定名称数组的元素
     * @param {string} name 指定名称
     * @param {RDomMakerMapItem} data 数据
     * @param {object} doms 用于存放元素的容器（一个对象）
     * @param {RDomMakerMakingSettings} settings 根节点
     */
    _createDom(name: string, data: RDomMakerMapItem, doms: object, settings: RDomMakerMakingSettings): JQuery;

    /**
     * 从容器中移除所有元素
     * @param {object} doms 存放元素的容器（一个对象）
     */
    removeAll(doms: object): this;
}

// #endregion

// #region RGlobalEvents

interface RGlobalEventsClass {
    /** 处理函数集合 */
    _eventlist: object;

    /**
     * 添加一个【鼠标移动事件】处理函数
     * @param {Function} event 要添加的处理函数
     */
    onMouseMove(event: Function): RGlobalEventsClass;

    /**
     * 添加一个【鼠标左键点下事件】处理函数
     * @param {Function} event 要添加的处理函数
     */
    onMouseDown(event: Function): RGlobalEventsClass;

    /**
     * 添加一个【鼠标左键松开事件】处理函数
     * @param {Function} event 要添加的处理函数
     */
    onMouseUp(event: Function): RGlobalEventsClass;

    /**
     * 添加一个【窗口大小改变事件】处理函数
     * @param {Function} event 要添加的处理函数
     */
    onResize(event: Function): RGlobalEventsClass;

    /**
     * 添加一个【键按下事件】处理函数
     * @param {Function} event 要添加的处理函数
     */
    onKeydown(event: Function): RGlobalEventsClass;

    /**
     * 添加一个【键放开事件】处理函数
     * @param {Function} event 要添加的处理函数
     */
    onKeyup(event: Function): RGlobalEventsClass;

    /**
     * 主动触发指定名称的事件
     * @param {string} name 指定名称
     * @param {any} arg 事件参数（同原生js事件参数）
     */
    triggerEvent(name: string, arg: any): RGlobalEventsClass;
}

/** 全局事件管理器 */
declare const RGlobalEvents: RGlobalEventsClass;

// #endregion

// #region RSolar

interface RSolarClass {
    /** 阳历节日数据 */
    festivals: object;

    /**
     * 获取指定阳历年的指定阳历月有几天
     * @param {number} year 指定阳历年
     * @param {number} month 指定阳历月
     */
    getDaysInMonth(year: number, month: number): number;

    /**
     * 获取用于日历显示的指定年月的日期数据
     * @param {number} year 指定阳历年
     * @param {number} month 指定阳历月
     */
    getDatesForCalendar(year: number, month: number): Array<RSolarDateData>;

    /**
     * 获取指定阳历月日的阳历节日
     * @param {number} month 指定阳历月
     * @param {number} date 指定阳历日
     */
    getFestivals(month: number, date: number): Array<string>;
}

interface RSolarDateData {
    /** 日期类型（1:这个月的日期 | 2:上个月的日期 | 3:下个月的日期） */
    type: number;
    /** 阳历年 */
    year: number;
    /** 阳历月 */
    month: number;
    /** 阳历日 */
    date: number;
}

/** 阳历工具 */
declare const RSolar: RSolarClass;

// #endregion

// #region RLunnar

interface RLunnarClass {
    /** 阴历节日数据 */
    festivals: object;

    /**
     * 将阳历日期转换为阴历日期
     * @param {number} year 阳历年
     * @param {number} month 阳历月
     * @param {number} date 阳历日
     */
    parse(year: number, month: number, date: number): RLunnarDate;

    /**
     * 获取指定阳历年的天干
     * @param {number} year 指定阳历年
     */
    gettg(year: number): string;

    /**
     * 获取指定阳历年的地支
     * @param {number} year 指定阳历年
     */
    getdz(year: number): string;

    /**
     * 获取指定阳历年的生肖
     * @param {number} year 指定阳历年
     */
    getsx(year: number): string;

    /**
     * 获取指定阳历年的生肖下标
     * @param {number} year 指定阳历年
     */
    getsxnum(year: number): number;

    /**
     * 获取指定阴历月的中文名称
     * @param {number} month 指定阴历月
     */
    getmonth(month: number): string;

    /**
     * 获取指定阴历日的中文名称
     * @param {number} date 指定阴历日
     */
    getdate(date: number): string;

    /**
     * 获取指定阴历月日的阴历节日
     * @param {number} month 指定阴历月
     * @param {number} date 指定阴历日
     */
    getFestivals(month: number, date: number): Array<string>;
}

/** 阴历日期 */
interface RLunnarDate {
    /** 阴历年 */
    year: number;
    /** 阴历月 */
    month: number;
    /** 阴历日 */
    date: number;
}

/** 阴历工具 */
declare const RLunnar: RLunnarClass;

// #endregion

// #region RAnimation

declare class RAnimation {
    /** Interval句柄 */
    handler: number;
    /** 动画开始时间 */
    startTime: number;
    /** 动画持续时间（毫秒数） */
    duration: number;
    /** 每一帧要执行的动作 */
    onFrame: (this: RAnimation, pastTime: number) => void;
    /** 每隔多久执行一帧（毫秒数） */
    interval: number;

    /**
     * 新建一个动画
     * @param {number} duration 动画持续时间
     * @param {(this: RAnimation, pastTime: number) => void} onFrame 每一帧要执行的动作
     */
    constructor(duration: number, onFrame: (this: RAnimation, pastTime: number) => void);

    /** 开始动画 */
    start(): void;

    /** 暂停动画 */
    pause(): void;

    /** 结束动画 */
    end(): void;
}

// #endregion

// #region StringReader

declare class StringReader {
    /** 要阅读的字符串 */
    str: string;

    /** 游标 */
    currentIndex: number;

    /** 当前游标所在的字符串 */
    ch: string;

    /** 判断当前游标是否已到达结尾 */
    isEnd(): boolean;

    /**
     * 向后移动游标指定位数
     * @param count 指定移动位数
     */
    move(count: number): void;

    /** 向后移动游标1位 */
    moveNext(): void;

    /**
     * 移动游标，向后查找指定的目标字符串
     * 直到目标字符串第一次出现或游标已到达结尾处为止
     * 返回是否找到
     * @param target 要查找的目标字符串
     */
    moveToString(target: string): boolean;

    /**
     * 不移动游标，向后查看指定个数个字符（不包含当前字符）
     * @param count 指定字符个数
     */
    peek(count: number): void;

    /**
     * 不移动游标，向后查看指定个数个字符（包含当前字符）
     * @param count 指定字符个数
     */
    look(count: number): void;
}

// #endregion

